package org.jabricks.lesson;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.border.Border;

import org.jabricks.jdialog.JBDialog;
import org.jabricks.jdialog.ComponentParams;
import org.osgi.framework.BundleContext;
import org.jabricks.resources.IResources;
import org.jabricks.resources.ResourcesImpl;
import org.jfree.chart.ChartPanel;

import java.util.Locale;
import java.util.ResourceBundle;

public class ChartForm extends JBDialog
{
	private static final long    serialVersionUID     = 1L;
	
	private        int           DEFAULT_WIDTH        = 420;
	private        int           DEFAULT_HEIGHT       = 315;

	private  JSplitPane          splitPane;
	private  final  int          DIVIDER_LOCATION     = 300;
	private  final  String       SPLITPANE            = "SPLITPANE";
 
	public   JButton             btnClose             = null;
	private  ResourceBundle      rsc                  = null;
    
	private  final   String      RESOURCES_btn_close  = "btn.close";
	private  final   String      RESOURCES            = "properties/properties";
 
    private  final   String[][]  names                = {{"30", "Временная, линейная" },
                                                         {"40", "Кусочно-линейная"   }};
    //-----------------------------------------------------
	public ChartForm(final BundleContext context)
	{
		super(context);

        IResources rsc = new ResourcesImpl();
        // Определение иконки
       	if (rsc != null)
       		setIconImage(rsc.getAppIcon());

		// Создание интерфейса
        this.setVisible(false);
        
    	readComponentParams();
		if (form_dim != null) {
			setSize(form_dim);
			DEFAULT_WIDTH  = form_dim.width;
			DEFAULT_HEIGHT = form_dim.height;
		}
		createGUI();

        // Получение панели содержимого
        Container container = getContentPane();
        
		JPanel buttons = createButtons();

        // Размещение панели desk
		container.add(pnlDesk, BorderLayout.CENTER);
        // Размещение панели с кнопками
        container.add(buttons, BorderLayout.SOUTH);

		// Локализация интерфейса
        changeLocale (locale);

       	// Расположение поверх остальных
		setAlwaysOnTop(true);

		setPreferredSize(new Dimension(DEFAULT_WIDTH,
				                       DEFAULT_HEIGHT));
		// Центрирование формы
		formCentering(DEFAULT_WIDTH, DEFAULT_HEIGHT);
    	this.setVisible(true);
	}
    //-----------------------------------------------------
	/**
	 * Создание интерфейса рабочей области формы
	 */
    private void createGUI()
    {
    	// Создание рабочей области
    	super.createDesk();
    	Border border;
    	border = BorderFactory.createEmptyBorder(2, 2, 2, 2);
    	pnlDesk.setBorder (border);

		splitPane = new JSplitPane();
		splitPane.setName(SPLITPANE);
        
		splitPane.setOneTouchExpandable(false);
		splitPane.setOrientation(JSplitPane.HORIZONTAL_SPLIT);

		
		if (cparams.size() == 0)
			splitPane.setDividerLocation(DIVIDER_LOCATION);
		else {
			ComponentParams cp;
			String   cls = JSplitPane.class.getSimpleName(); 
			cp = extractComponentParams(cls, SPLITPANE);
			if (cp != null)
				splitPane.setDividerLocation(cp.getLocation());
			else
				splitPane.setDividerLocation(DIVIDER_LOCATION);
		}
    	pnlDesk.setLayout(new BorderLayout());
		pnlDesk.add(splitPane, BorderLayout.CENTER);
		
		ChartPanel pnlChart;
		pnlChart = (new ModuleDAO()).createTimeSeriesChart();
		
		splitPane.setLeftComponent (pnlChart);		
		splitPane.setRightComponent(createFormPanel());
    }
    //-----------------------------------------------------
    private JPanel createFormPanel()
    {
    	JPanel panel = new JPanel();
    	
    	ActionListener listener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				JRadioButton radio;
				radio = (JRadioButton) event.getSource();
				selectDiagram(radio.getName());
			}
    	};
    	
        // Группа связанных радио-переключателей
    	JPanel panelRadio = new JPanel(new GridLayout(0, 1, 0, 5));
        panelRadio.setBorder(BorderFactory.createTitledBorder("Диаграммы"));
        ButtonGroup bg = new ButtonGroup();
        for (int i = 0; i < names.length; i++) {
            JRadioButton radio = new JRadioButton(names[i][1]);
            radio.setName(names[i][0]);
           	radio.setSelected(i == 0);
            radio.addActionListener(listener);
            panelRadio.add(radio);
            bg.add(radio);
        }
        
        panel.add(panelRadio);
    	return panel;
    }
    //-----------------------------------------------------
	/**
	 * Создание панели кнопок управления
	 * @return панели управления JPanel
	 */
    private JPanel createButtons()
    {
        String caption = EMPTY;
        // Кнопка Закрыть
        btnClose = new JButton(caption);
        Dimension dim = new Dimension(BUTTON_width, 
        		                      BUTTON_height);
        btnClose.setPreferredSize(dim);
        /*
         *  Создание панели 'pnlButtons' с последовательным
         *  расположением компонентов и выравниванием по 
         *  правому краю 'flow'
         */
        JPanel flow = super.createButtonsPanel(); 
        pnlButtons.add (btnClose);

        return flow;
    }
    //-----------------------------------------------------
    /**
     * Процедура локализации интерфейса
     * @param locale объект локализации
     */
    public void changeLocale (Locale locale)
    {
		rsc = ResourceBundle.getBundle (RESOURCES, locale);

		String close   = RESOURCES_btn_close;
		String caption = RESOURCE_title;
		
		if (rsc != null) {
			caption = rsc.getString(RESOURCE_title);
			close  = SPAN_normal + rsc.getString(close);
		}
		this.setTitle(caption);
		btnClose.setText(close);
    }
    //-----------------------------------------------------
    private void selectDiagram(final String name)
    {
		int  location = splitPane.getDividerLocation(); 
    	
		ChartPanel cp = null;
		if (name.equals(names[0][0]))
    		cp = (new ModuleDAO()).createTimeSeriesChart();
    	else if (name.equals(names[1][0]))
    		cp = (new ModuleDAO()).createLineChart();
		
		if (cp != null) {
			splitPane.setLeftComponent (cp);
			splitPane.setDividerLocation(location);
		}
    }
    //-----------------------------------------------------
}
