package org.jabricks.lesson;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.awt.Color;
import java.util.ArrayList;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;

import org.jfree.data.Range;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesDataItem;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYDataItem;

import org.jabricks.jfreechart.ChartDescTime;
import org.jabricks.jfreechart.ChartDescXY;
import org.jabricks.jfreechart.JFreeChartImpl;

public class ModuleDAO
{
	private final String[]    TIMESERIES = {
            "Курс USD", 
            "Курс EUR", 
            "Нефть марки Brent"};
	private final String[][]  DATA_TIMESERIES = {
            // Дата            USD        EUR  Нефть марки Brent
			{"2017-05-11", "58.0824", "63.2634", "50.78"},
			{"2017-05-12", "57.1161", "62.1595", "50.82"},
			{"2017-05-13", "57.1640", "62.0915", "51.77"},
			{"2017-05-16", "56.5258", "61.8449", "51.30"},
			{"2017-05-17", "56.2603", "62.0382", "52.14"}};
	//-----------------------------------------------------
	public ModuleDAO(){}
	//-----------------------------------------------------
	private List<TimeSeriesDataItem> createCollectionTM(final int idx)
	{
		List<TimeSeriesDataItem> data;
		data = new ArrayList<TimeSeriesDataItem>();

		TimeSeriesDataItem tsdi;
		Date               date;
		SimpleDateFormat   sdf ;
 		sdf = new SimpleDateFormat("yyyy-MM-dd");
		try {
			for (int i = 0; i < DATA_TIMESERIES.length; i++){

				date    = sdf.parse (DATA_TIMESERIES[i][0]);
				Day    day = new Day(date);
				
				String vl  = DATA_TIMESERIES[i][idx];
				
				double val = Double.valueOf(vl);
					
				tsdi = new TimeSeriesDataItem(day, val);
				data.add(tsdi);
			}
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return data;
	}
	//-----------------------------------------------------
	private XYDataset createXYDataset()
	{
		List<TimeSeriesDataItem> data_usd;
		List<TimeSeriesDataItem> data_eur;
		List<TimeSeriesDataItem> data_oil;

		TimeSeries               s_usd;
		TimeSeries               s_eur;
		TimeSeries               s_oil;

		data_usd = createCollectionTM(1);
		data_eur = createCollectionTM(2);
		data_oil = createCollectionTM(3);

		JFreeChartImpl fci = new JFreeChartImpl();
		
		s_usd = fci.createTimeSeries(TIMESERIES[0], data_usd);
		s_eur = fci.createTimeSeries(TIMESERIES[1], data_eur);
		s_oil = fci.createTimeSeries(TIMESERIES[2], data_oil);

		return fci.createXYDataset(s_usd, s_eur, s_oil);
	}
	//-----------------------------------------------------
	private List<XYDataItem> createCollectionXY(final int idx)
	{
		List<XYDataItem> data;
		data = new ArrayList<XYDataItem>();

		for (int i = 0; i < DATA_TIMESERIES.length; i++){
			
			String dd = DATA_TIMESERIES[i][0].substring(8);
			String vl = DATA_TIMESERIES[i][idx];
			
			double day = Double.valueOf(dd);
			double val = Double.valueOf(vl);
				
			XYDataItem rxy = new XYDataItem(day, val);
			data.add(rxy);
		}
		return data;
	}
	//-----------------------------------------------------
	public XYDataset createDatasetXY()
	{
		List<XYDataItem> data_usd;
		List<XYDataItem> data_eur;
		List<XYDataItem> data_oil;

		XYSeries         s_usd   ;
		XYSeries         s_eur   ;
		XYSeries         s_oil   ;

		data_usd = createCollectionXY(1);
		data_eur = createCollectionXY(2);
		data_oil = createCollectionXY(3);
		
		JFreeChartImpl fci = new JFreeChartImpl();
		
		s_usd = fci.createXYSeries(TIMESERIES[0], data_usd);
		s_eur = fci.createXYSeries(TIMESERIES[1], data_eur);
		s_oil = fci.createXYSeries(TIMESERIES[2], data_oil);
		
    	return fci.createXYDataset(s_usd, s_eur, s_oil);
	}
	//-----------------------------------------------------
	public ChartPanel createTimeSeriesChart()
	{
		 XYDataset dataset = createXYDataset();
	    	
		 ChartDescTime cdt = new ChartDescTime();
		 cdt.setTitle   ("Временная (линейная) диаграмма");
		 cdt.setSubtitle("Валюта, нефть с 11.05.2017 по 25.05.2017");
		 cdt.setDataset (dataset);
		 cdt.setLegend  (true   );
		 cdt.setTooltips(true   );

//		 cdt.setSeriesShapeVisible(true);
		 cdt.setAxisDateFormat(new SimpleDateFormat("dd.MM"));

		 JFreeChartImpl fci = new JFreeChartImpl();
		 JFreeChart chart = fci.createTimeSeriesChart(cdt);
		 
		 fci.setSeriesPaint(chart, 2, Color.MAGENTA, 2f); 
         return new ChartPanel(chart);
	}
	//-----------------------------------------------------
	public ChartPanel createLineChart()
	{
		XYDataset dataset = createDatasetXY();
		
		ChartDescXY cdxy = new ChartDescXY();
		cdxy.setTitle   ("Кусочно-линейная диаграмма");
		cdxy.setSubtitle("Валюта, нефть с 11.05.2017 по 25.05.2017");
		cdxy.setDataset (dataset);
		cdxy.setLegend  (true   );
		cdxy.setTooltips(true   );
		
		cdxy.setRange(new Range(50, 65));

		cdxy.setYTickUnit(NumberAxis.createIntegerTickUnits());
		cdxy.setXTickUnit(NumberAxis.createIntegerTickUnits());

		cdxy.setAxisYVisible(false);
		cdxy.setAxisXVisible(false);

		cdxy.setSeriesShapeVisible(true);

		JFreeChartImpl fci = new JFreeChartImpl();
//		fci.setSeriesPaint(chart, 2, Color.MAGENTA, 2f); 

		JFreeChart chart = fci.createLineChart(cdxy);
		 
        return new ChartPanel(chart);
		
/*
//        XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
//        // Удаление связующих линий Series 1
//      renderer.setSeriesLinesVisible (0, false);
//        // Удаление меток Series 2
//      renderer.setSeriesShapesVisible(1, false);
*/        
	}
	//-----------------------------------------------------
}
